import React, { useState, useEffect } from 'react';
import { useNavigate } from 'react-router-dom';
import { Plus, Upload, Lock, Trash2, X } from 'lucide-react';
import type { Match } from '../types';

// Add this interface for uploaded files
interface UploadedFile {
  file: File;
  preview: string;
  extractedDirection: string | null;
  extractedSeatNumber: number | null;
  manualSeatNumber: number | null;
}

function AdminPage() {
  const navigate = useNavigate();
  const [isAuthenticated, setIsAuthenticated] = useState(() => {
    // Check if we're already authenticated from a previous session
    return localStorage.getItem('isAdminAuthenticated') === 'true';
  });
  const [isFirstTime, setIsFirstTime] = useState(false);
  const [isLoading, setIsLoading] = useState(true);
  const [password, setPassword] = useState('');
  const [newPassword, setNewPassword] = useState('');
  const [confirmPassword, setConfirmPassword] = useState('');
  const [error, setError] = useState('');
  
  const [matchData, setMatchData] = useState({
    name: '',
    date: '',
    location: '',
    totalSeats: 0,
    price: 0,
    timeoutDate: '',
  });
  const [uploadedFiles, setUploadedFiles] = useState<UploadedFile[]>([]);
  const [isSubmitting, setIsSubmitting] = useState(false);
  const [matches, setMatches] = useState<Match[]>([]);

  useEffect(() => {
    const checkPassword = async () => {
      // Only check if we're not already authenticated
      if (isAuthenticated) {
        setIsLoading(false);
        return;
      }

      try {
        setIsLoading(true);
        setError('');
        const response = await fetch('/api/admin/check-password');
        if (!response.ok) {
          throw new Error('Failed to check password status');
        }
        const data = await response.json();
        setIsFirstTime(data.isFirstTime);
      } catch (error) {
        console.error('Error checking password:', error);
        setError('Failed to check password status. Please try again later.');
      } finally {
        setIsLoading(false);
      }
    };
    checkPassword();
  }, [isAuthenticated]);

  useEffect(() => {
    const fetchMatches = async () => {
      if (!isAuthenticated) return;
      try {
        const response = await fetch('/api/matches');
        if (!response.ok) {
          throw new Error('Failed to fetch matches');
        }
        const data = await response.json();
        setMatches(data);
      } catch (error) {
        console.error('Error fetching matches:', error);
      }
    };
    fetchMatches();
  }, [isAuthenticated]);

  const handleLogin = async (e: React.FormEvent) => {
    e.preventDefault();
    setError('');

    if (isFirstTime) {
      if (newPassword !== confirmPassword) {
        setError('Passwords do not match');
        return;
      }
      try {
        const response = await fetch('/api/admin/set-password', {
          method: 'POST',
          headers: {
            'Content-Type': 'application/json',
          },
          body: JSON.stringify({ password: newPassword }),
        });
        
        if (!response.ok) {
          const data = await response.json();
          throw new Error(data.message || 'Failed to set password');
        }
        
        setIsAuthenticated(true);
        localStorage.setItem('isAdminAuthenticated', 'true');
        setNewPassword('');
        setConfirmPassword('');
      } catch (error: any) {
        setError(error.message);
      }
    } else {
      try {
        const response = await fetch('/api/admin/verify-password', {
          method: 'POST',
          headers: {
            'Content-Type': 'application/json',
          },
          body: JSON.stringify({ password }),
        });
        
        if (!response.ok) {
          const data = await response.json();
          throw new Error(data.message || 'Invalid password');
        }
        
        setIsAuthenticated(true);
        localStorage.setItem('isAdminAuthenticated', 'true');
        setPassword('');
      } catch (error: any) {
        setError(error.message);
      }
    }
  };

  const handleLogout = () => {
    setIsAuthenticated(false);
    localStorage.removeItem('isAdminAuthenticated');
  };

  const handleRemoveFile = (index: number) => {
    setUploadedFiles(prev => prev.filter((_, i) => i !== index));
  };

  const handleFileChange = async (e: React.ChangeEvent<HTMLInputElement>) => {
    const files = Array.from(e.target.files || []);
    const newFiles: UploadedFile[] = await Promise.all(files.map(async file => {
      // Perform filename parsing on the frontend
      const extractedInfo = await extractSeatInfoFromFilenameFrontend(file.name);
      return {
        file,
        preview: URL.createObjectURL(file),
        extractedDirection: extractedInfo.direction,
        extractedSeatNumber: extractedInfo.extractedSeatNumber,
        manualSeatNumber: extractedInfo.extractedSeatNumber, // Initialize manual with extracted
      };
    }));
    setUploadedFiles(prev => [...prev, ...newFiles]);
  };

  // Helper function for frontend filename parsing (matches backend logic)
  const extractSeatInfoFromFilenameFrontend = async (filename: string) => {
    // Look for 3 to 5 uppercase letters, followed by an underscore, then numbers, ending with .pdf
    const regex = /^([A-Z]{3,5})_(\d+)\.pdf$/;
    const match = filename.match(regex);
    if (match && match[1] && match[2]) {
      return {
        direction: match[1],
        extractedSeatNumber: parseInt(match[2], 10)
      };
    } else {
      return { direction: null, extractedSeatNumber: null };
    }
  };

  const handleManualSeatNumberChange = (index: number, value: string) => {
    const numberValue = parseInt(value, 10);
    setUploadedFiles(prev =>
      prev.map((item, i) =>
        i === index ? { ...item, manualSeatNumber: isNaN(numberValue) ? null : numberValue } : item
      )
    );
  };

  const handleSubmit = async (e: React.FormEvent) => {
    e.preventDefault();
    if (!isAuthenticated) return;

    const formData = new FormData();
    formData.append('name', matchData.name);
    formData.append('date', matchData.date);
    formData.append('location', matchData.location);
    formData.append('totalSeats', matchData.totalSeats.toString());
    formData.append('price', matchData.price.toString());
    formData.append('timeoutDate', matchData.timeoutDate);

    // Append only the files that haven't been removed
    // Also send the manual seat number along with the file
    uploadedFiles.forEach(({ file, manualSeatNumber, extractedDirection, extractedSeatNumber }) => {
      formData.append('pdfFiles', file);
      // Append the relevant seat info for this file
      formData.append('seatInfo', JSON.stringify({
        originalFilename: file.name,
        direction: extractedDirection,
        extractedSeatNumber: extractedSeatNumber,
        manualSeatNumber: manualSeatNumber,
      }));
    });

    try {
      const response = await fetch('/api/admin/matches', {
        method: 'POST',
        headers: {
          'Authorization': `Bearer ${localStorage.getItem('adminToken')}`
        },
        body: formData
      });

      if (!response.ok) {
        throw new Error('Failed to add match');
      }

      // Clear the form and uploaded files
      setMatchData({
        name: '',
        date: '',
        location: '',
        totalSeats: 0,
        price: 0,
        timeoutDate: ''
      });
      setUploadedFiles([]);
      
      // Refresh matches list
      fetchMatches();
    } catch (error) {
      console.error('Error adding match:', error);
    }
  };

  // Define fetchMatches here so it can be called from handleSubmit and useEffect
  const fetchMatches = async () => {
    if (!isAuthenticated) return;
    try {
      const response = await fetch('/api/matches');
      if (!response.ok) {
        throw new Error('Failed to fetch matches');
      }
      const data = await response.json();
      setMatches(data);
    } catch (error) {
      console.error('Error fetching matches:', error);
    }
  };

  // Function to handle match deletion
  const handleDeleteMatch = async (matchId: number) => {
    if (window.confirm('Are you sure you want to delete this match? This action cannot be undone.')) {
      try {
        const response = await fetch(`/api/admin/matches/${matchId}`, {
          method: 'DELETE',
          headers: {
            'Authorization': `Bearer ${localStorage.getItem('adminToken')}`
          }
        });

        if (!response.ok) {
          const data = await response.json();
          throw new Error(data.message || 'Failed to delete match');
        }

        alert('Match deleted successfully!');
        // Refetch matches after deletion to update the list
        fetchMatches();
      } catch (error: any) {
        console.error('Error deleting match:', error);
        alert(`Failed to delete match: ${error.message}`);
      }
    }
  };

  // Cleanup file previews
  useEffect(() => {
    return () => {
      uploadedFiles.forEach(({ preview }) => {
        URL.revokeObjectURL(preview);
      });
    };
  }, [uploadedFiles]);

  if (isLoading) {
    return (
      <div className="flex items-center justify-center min-h-[60vh]">
        <div className="animate-spin rounded-full h-12 w-12 border-t-2 border-b-2 border-blue-500"></div>
      </div>
    );
  }

  if (!isAuthenticated) {
    return (
      <div className="max-w-md mx-auto">
        <div className="bg-gray-800 p-8 rounded-lg shadow-lg">
          <div className="flex items-center justify-center mb-6">
            <Lock className="w-12 h-12 text-blue-500" />
          </div>
          <h2 className="text-2xl font-bold mb-6 text-center">
            {isFirstTime ? 'Set Admin Password' : 'Admin Login'}
          </h2>
          
          <form onSubmit={handleLogin} className="space-y-4">
            {isFirstTime ? (
              <>
                <div>
                  <label className="block mb-1">New Password</label>
                  <input
                    type="password"
                    value={newPassword}
                    onChange={(e) => setNewPassword(e.target.value)}
                    className="w-full bg-gray-700 rounded px-3 py-2"
                    required
                    minLength={8}
                  />
                </div>
                <div>
                  <label className="block mb-1">Confirm Password</label>
                  <input
                    type="password"
                    value={confirmPassword}
                    onChange={(e) => setConfirmPassword(e.target.value)}
                    className="w-full bg-gray-700 rounded px-3 py-2"
                    required
                    minLength={8}
                  />
                </div>
              </>
            ) : (
              <div>
                <label className="block mb-1">Password</label>
                <input
                  type="password"
                  value={password}
                  onChange={(e) => setPassword(e.target.value)}
                  className="w-full bg-gray-700 rounded px-3 py-2"
                  required
                />
              </div>
            )}
            
            {error && (
              <p className="text-red-500 text-sm">{error}</p>
            )}
            
            <button
              type="submit"
              className="w-full bg-blue-600 hover:bg-blue-700 text-white font-bold py-2 px-4 rounded"
            >
              {isFirstTime ? 'Set Password' : 'Login'}
            </button>
          </form>
        </div>
      </div>
    );
  }

  return (
    <div className="max-w-4xl mx-auto">
      <div className="flex justify-between items-center mb-8">
        <h1 className="text-3xl font-bold">Admin Dashboard</h1>
        <button
          onClick={handleLogout}
          className="bg-red-600 hover:bg-red-700 px-4 py-2 rounded"
        >
          Logout
        </button>
      </div>

      <div className="grid gap-8">
        <form onSubmit={handleSubmit} className="bg-gray-800 p-6 rounded-lg shadow-lg">
          <h2 className="text-xl font-bold mb-6">Create New Match</h2>
          
          {error && (
            <div className="bg-red-500 text-white p-4 rounded-lg mb-4">
              {error}
            </div>
          )}
          
          <div className="grid grid-cols-2 gap-6">
            <div>
              <label className="block mb-2">Match Name</label>
              <input
                type="text"
                value={matchData.name}
                onChange={(e) => setMatchData({ ...matchData, name: e.target.value })}
                className="w-full bg-gray-700 rounded px-3 py-2"
                required
              />
            </div>
            
            <div>
              <label className="block mb-2">Date & Time</label>
              <input
                type="datetime-local"
                value={matchData.date}
                onChange={(e) => setMatchData({ ...matchData, date: e.target.value })}
                className="w-full bg-gray-700 rounded px-3 py-2"
                required
              />
            </div>

            <div>
              <label className="block mb-2">Location</label>
              <input
                type="text"
                value={matchData.location}
                onChange={(e) => setMatchData({ ...matchData, location: e.target.value })}
                className="w-full bg-gray-700 rounded px-3 py-2"
                required
              />
            </div>

            <div>
              <label className="block mb-2">Total Seats</label>
              <input
                type="number"
                value={matchData.totalSeats}
                onChange={(e) => setMatchData({ ...matchData, totalSeats: parseInt(e.target.value) || 0 })}
                className="w-full bg-gray-700 rounded px-3 py-2"
                required
                min="1"
              />
            </div>

            <div>
              <label className="block mb-2">Price (€)</label>
              <input
                type="number"
                value={matchData.price}
                onChange={(e) => setMatchData({ ...matchData, price: parseFloat(e.target.value) || 0 })}
                className="w-full bg-gray-700 rounded px-3 py-2"
                min="0"
                step="0.01"
              />
            </div>

            <div>
              <label className="block mb-2">Booking Deadline</label>
              <input
                type="datetime-local"
                value={matchData.timeoutDate}
                onChange={(e) => setMatchData({ ...matchData, timeoutDate: e.target.value })}
                className="w-full bg-gray-700 rounded px-3 py-2"
                required
              />
            </div>
          </div>

          <div className="mt-6">
            <label className="block mb-2">PDF Files</label>
            <input
              type="file"
              accept=".pdf"
              multiple
              onChange={handleFileChange}
              className="w-full bg-gray-700 rounded px-3 py-2"
            />
            {uploadedFiles.length > 0 && (
              <div className="mt-2 space-y-2">
                {uploadedFiles.map((file, index) => (
                  <div key={index} className="flex items-center justify-between bg-gray-700 p-2 rounded">
                    <div className="flex-1 truncate mr-4">
                      <span className="text-sm block">{file.file.name}</span>
                      {file.extractedDirection && file.extractedSeatNumber && (
                        <span className="text-xs text-gray-400">Extracted: {file.extractedDirection} {file.extractedSeatNumber}</span>
                      )}
                    </div>
                    <div className="flex items-center">
                       <label className="text-sm mr-2">Seat No:</label>
                       <input
                         type="number"
                         value={file.manualSeatNumber !== null ? file.manualSeatNumber : ''}
                         onChange={(e) => handleManualSeatNumberChange(index, e.target.value)}
                         className="w-20 bg-gray-600 rounded px-2 py-1 text-sm"
                         min="1"
                       />
                    </div>
                    <button
                      type="button"
                      onClick={() => handleRemoveFile(index)}
                      className="text-red-500 hover:text-red-400 ml-4"
                    >
                      <X className="w-4 h-4" />
                    </button>
                  </div>
                ))}
              </div>
            )}
          </div>

          <button
            type="submit"
            disabled={isSubmitting}
            className={`mt-8 w-full bg-blue-600 hover:bg-blue-700 text-white font-bold py-3 px-6 rounded-lg flex items-center justify-center gap-2 ${
              isSubmitting ? 'opacity-50 cursor-not-allowed' : ''
            }`}
          >
            {isSubmitting ? (
              <>
                <div className="animate-spin rounded-full h-5 w-5 border-t-2 border-b-2 border-white"></div>
                Creating Match...
              </>
            ) : (
              <>
                <Plus className="w-5 h-5" />
                Create Match
              </>
            )}
          </button>
        </form>
      </div>

      {/* Existing Matches List */}
      <div className="mt-12">
        <h2 className="text-2xl font-bold mb-4">Existing Matches</h2>
        <div className="bg-gray-800 rounded-lg shadow-lg p-6">
          {matches.length === 0 ? (
            <p>No matches found.</p>
          ) : (
            <ul className="space-y-4">
              {matches.map((match) => (
                <li key={match.id} className="border-b border-gray-700 pb-4 last:border-b-0 flex justify-between items-center">
                  <div>
                    <h3 className="text-xl font-semibold">{match.name}</h3>
                    <p className="text-sm text-gray-400">{new Date(match.date).toLocaleString()}</p>
                    <p className="text-sm text-gray-400">{match.location}</p>
                  </div>
                  <button
                    onClick={() => handleDeleteMatch(match.id)}
                    className="bg-red-600 hover:bg-red-700 text-white px-3 py-1 rounded flex items-center gap-1"
                  >
                    <Trash2 className="w-4 h-4" />
                    Remove
                  </button>
                </li>
              ))}
            </ul>
          )}
        </div>
      </div>
    </div>
  );
}

export default AdminPage;