import { useState, useEffect } from 'react';
import { Link } from 'react-router-dom';
import { Calendar, MapPin, Users, Loader2, Mail } from 'lucide-react';
import { api, type Match } from '../services/api';

function HomePage() {
  const [matches, setMatches] = useState<Match[]>([]);
  const [isLoading, setIsLoading] = useState(true);
  const [error, setError] = useState<string | null>(null);

  useEffect(() => {
    const loadMatches = async () => {
      try {
        setIsLoading(true);
        setError(null);
        const data = await api.matches.getAll();
        setMatches(data);
      } catch (error) {
        console.error('Error fetching matches:', error);
        setError('Failed to load matches. Please try again later.');
      } finally {
        setIsLoading(false);
      }
    };
    loadMatches();
  }, []);

  if (isLoading) {
    return (
      <div className="flex items-center justify-center min-h-[60vh]">
        <Loader2 className="w-8 h-8 animate-spin text-blue-500" />
      </div>
    );
  }

  if (error) {
    return (
      <div className="text-center py-12">
        <h2 className="text-2xl font-bold text-red-500 mb-4">Error</h2>
        <p className="text-gray-400">{error}</p>
        <button 
          onClick={() => window.location.reload()} 
          className="mt-4 bg-blue-600 hover:bg-blue-700 px-4 py-2 rounded-lg"
        >
          Try Again
        </button>
      </div>
    );
  }

  if (matches.length === 0) {
    return (
      <div className="text-center py-12">
        <h2 className="text-2xl font-bold mb-4">No Matches Available</h2>
        <p className="text-gray-400 mb-6">There are no upcoming matches at the moment.</p>
      </div>
    );
  }

  return (
    <div>
      <h1 className="text-4xl font-bold mb-8 text-center">Upcoming Handball Matches</h1>
      
      <div className="grid grid-cols-1 md:grid-cols-2 lg:grid-cols-3 gap-6">
        {matches.map((match) => (
          <Link
            key={match.id}
            to={`/match/${match.id}`}
            className="bg-gray-800 rounded-lg overflow-hidden hover:bg-gray-700 transition-colors"
          >
            <div className="p-6">
              <h2 className="text-xl font-bold mb-4">{match.name}</h2>
              
              <div className="space-y-3 text-gray-300">
                <div className="flex items-center gap-2">
                  <Calendar className="w-5 h-5" />
                  <span>{new Date(match.date).toLocaleDateString('fr-FR', {
                    day: 'numeric',
                    month: 'long',
                    year: 'numeric',
                    hour: '2-digit',
                    minute: '2-digit'
                  })}</span>
                </div>

                <div className="flex items-center gap-2">
                  <MapPin className="w-5 h-5" />
                  <span>{match.location}</span>
                </div>

                <div className="flex items-center gap-2">
                  <Users className="w-5 h-5" />
                  <span>{match.availableSeats} seats available</span>
                </div>
              </div>

              <div className="mt-6 flex items-center justify-between">
                {match.price > 0 && <span className="text-2xl font-bold">{match.price}€</span>}
                <button className="bg-blue-600 hover:bg-blue-700 px-4 py-2 rounded-lg">
                  Book Now
                </button>
              </div>
            </div>
          </Link>
        ))}
      </div>

      <footer className="mt-12 text-center text-gray-400">
        <div className="flex items-center justify-center gap-2">
          <Mail className="w-5 h-5" />
          <a href="mailto:alexandreyagoubi@gmail.com" className="hover:text-gray-300">
            alexandreyagoubi@gmail.com
          </a>
        </div>
      </footer>
    </div>
  );
}

export default HomePage;